@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2012 OpenStack LLC
+# Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -14,13 +14,14 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import os.path
+
 import ldap
 from ldap import filter as ldap_filter
 
 from keystone.common.ldap import fakeldap
-from keystone.common import logging
 from keystone import exception
-
+from keystone.openstack.common import log as logging
 
 LOG = logging.getLogger(__name__)
 
@@ -34,6 +35,9 @@ LDAP_DEREF = {'always': ldap.DEREF_ALWAYS,
               'finding': ldap.DEREF_FINDING,
               'never': ldap.DEREF_NEVER,
               'searching': ldap.DEREF_SEARCHING}
+LDAP_TLS_CERTS = {'never': ldap.OPT_X_TLS_NEVER,
+                  'demand': ldap.OPT_X_TLS_DEMAND,
+                  'allow': ldap.OPT_X_TLS_ALLOW}
 
 
 def py2ldap(val):
@@ -75,12 +79,25 @@ def parse_deref(opt):
                          opt) + ', '.join(LDAP_DEREF.keys()))
 
 
+def parse_tls_cert(opt):
+    try:
+        return LDAP_TLS_CERTS[opt]
+    except KeyError:
+        raise ValueError(_(
+            'Invalid LDAP TLS certs option: %(option)s. '
+            'Choose one of: %(options)s') % {
+                'option': opt,
+                'options': ', '.join(LDAP_TLS_CERTS.keys())})
+
+
 def ldap_scope(scope):
     try:
         return LDAP_SCOPES[scope]
     except KeyError:
-        raise ValueError(_('Invalid LDAP scope: %s. Choose one of: ' % scope) +
-                         ', '.join(LDAP_SCOPES.keys()))
+        raise ValueError(
+            _('Invalid LDAP scope: %(scope)s. Choose one of: %(options)s') % {
+                'scope': scope,
+                'options': ', '.join(LDAP_SCOPES.keys())})
 
 
 class BaseLdap(object):
@@ -90,12 +107,14 @@ class BaseLdap(object):
     DEFAULT_ID_ATTR = 'cn'
     DEFAULT_OBJECTCLASS = None
     DEFAULT_FILTER = None
+    DEFAULT_EXTRA_ATTR_MAPPING = []
     DUMB_MEMBER_DN = 'cn=dumb,dc=nonexistent'
     NotFound = None
     notfound_arg = None
     options_name = None
     model = None
-    attribute_mapping = {}
+    attribute_options_names = {}
+    immutable_attrs = []
     attribute_ignore = []
     tree_dn = None
 
@@ -106,6 +125,11 @@ class BaseLdap(object):
         self.LDAP_SCOPE = ldap_scope(conf.ldap.query_scope)
         self.alias_dereferencing = parse_deref(conf.ldap.alias_dereferencing)
         self.page_size = conf.ldap.page_size
+        self.use_tls = conf.ldap.use_tls
+        self.tls_cacertfile = conf.ldap.tls_cacertfile
+        self.tls_cacertdir = conf.ldap.tls_cacertdir
+        self.tls_req_cert = parse_tls_cert(conf.ldap.tls_req_cert)
+        self.attribute_mapping = {}
 
         if self.options_name is not None:
             self.suffix = conf.ldap.suffix
@@ -122,6 +146,16 @@ class BaseLdap(object):
             self.object_class = (getattr(conf.ldap, objclass)
                                  or self.DEFAULT_OBJECTCLASS)
 
+            for k, v in self.attribute_options_names.iteritems():
+                v = '%s_%s_attribute' % (self.options_name, v)
+                self.attribute_mapping[k] = getattr(conf.ldap, v)
+
+            attr_mapping_opt = ('%s_additional_attribute_mapping' %
+                                self.options_name)
+            attr_mapping = (getattr(conf.ldap, attr_mapping_opt)
+                            or self.DEFAULT_EXTRA_ATTR_MAPPING)
+            self.extra_attr_mapping = self._parse_extra_attrs(attr_mapping)
+
             filter = '%s_filter' % self.options_name
             self.filter = getattr(conf.ldap, filter) or self.DEFAULT_FILTER
 
@@ -138,6 +172,10 @@ class BaseLdap(object):
 
             if self.notfound_arg is None:
                 self.notfound_arg = self.options_name + '_id'
+
+            attribute_ignore = '%s_attribute_ignore' % self.options_name
+            self.attribute_ignore = getattr(conf.ldap, attribute_ignore)
+
         self.use_dumb_member = getattr(conf.ldap, 'use_dumb_member')
         self.dumb_member = (getattr(conf.ldap, 'dumb_member') or
                             self.DUMB_MEMBER_DN)
@@ -151,13 +189,37 @@ class BaseLdap(object):
         else:
             return self.NotFound(**{self.notfound_arg: object_id})
 
+    def _parse_extra_attrs(self, option_list):
+        mapping = {}
+        for item in option_list:
+            try:
+                ldap_attr, attr_map = item.split(':')
+            except Exception:
+                LOG.warn(_(
+                    'Invalid additional attribute mapping: "%s". '
+                    'Format must be <ldap_attribute>:<keystone_attribute>')
+                    % item)
+                continue
+            if attr_map not in self.attribute_mapping:
+                LOG.warn(_('Invalid additional attribute mapping: "%(item)s". '
+                           'Value "%(attr_map)s" must use one of %(keys)s.') %
+                         {'item': item, 'attr_map': attr_map,
+                          'keys': ', '.join(self.attribute_mapping.keys())})
+                continue
+            mapping[ldap_attr] = attr_map
+        return mapping
+
     def get_connection(self, user=None, password=None):
         if self.LDAP_URL.startswith('fake://'):
             conn = fakeldap.FakeLdap(self.LDAP_URL)
         else:
             conn = LdapWrapper(self.LDAP_URL,
                                self.page_size,
-                               alias_dereferencing=self.alias_dereferencing)
+                               alias_dereferencing=self.alias_dereferencing,
+                               use_tls=self.use_tls,
+                               tls_cacertfile=self.tls_cacertfile,
+                               tls_cacertdir=self.tls_cacertdir,
+                               tls_req_cert=self.tls_req_cert)
 
         if user is None:
             user = self.LDAP_USER
@@ -181,12 +243,15 @@ class BaseLdap(object):
         if self.LDAP_SCOPE == ldap.SCOPE_ONELEVEL:
             return self._id_to_dn_string(id)
         conn = self.get_connection()
-        search_result = conn.search_s(
-            self.tree_dn, self.LDAP_SCOPE,
-            '(&(%(id_attr)s=%(id)s)(objectclass=%(objclass)s))' %
-            {'id_attr': self.id_attr,
-             'id': ldap.filter.escape_filter_chars(str(id)),
-             'objclass': self.object_class})
+        try:
+            search_result = conn.search_s(
+                self.tree_dn, self.LDAP_SCOPE,
+                '(&(%(id_attr)s=%(id)s)(objectclass=%(objclass)s))' %
+                {'id_attr': self.id_attr,
+                 'id': ldap.filter.escape_filter_chars(str(id)),
+                 'objclass': self.object_class})
+        finally:
+            conn.unbind_s()
         if search_result:
             dn, attrs = search_result[0]
             return dn
@@ -237,6 +302,7 @@ class BaseLdap(object):
                                          values['id'])
 
     def create(self, values):
+        self.affirm_unique(values)
         if not self.allow_create:
             action = _('LDAP %s create') % self.options_name
             raise exception.ForbiddenAction(action=action)
@@ -249,12 +315,20 @@ class BaseLdap(object):
                 continue
             if v is not None:
                 attr_type = self.attribute_mapping.get(k, k)
-                attrs.append((attr_type, [v]))
+                if attr_type is not None:
+                    attrs.append((attr_type, [v]))
+                extra_attrs = [attr for attr, name
+                               in self.extra_attr_mapping.iteritems()
+                               if name == k]
+                for attr in extra_attrs:
+                    attrs.append((attr, [v]))
 
         if 'groupOfNames' in object_classes and self.use_dumb_member:
             attrs.append(('member', [self.dumb_member]))
-
-        conn.add_s(self._id_to_dn(values['id']), attrs)
+        try:
+            conn.add_s(self._id_to_dn(values['id']), attrs)
+        finally:
+            conn.unbind_s()
         return values
 
     def _ldap_get(self, id, filter=None):
@@ -267,10 +341,13 @@ class BaseLdap(object):
                     'filter': (filter or self.filter or ''),
                     'object_class': self.object_class})
         try:
-            res = conn.search_s(self.tree_dn, self.LDAP_SCOPE, query,
-                                self.attribute_mapping.values())
+            attrs = list(set((self.attribute_mapping.values() +
+                              self.extra_attr_mapping.keys())))
+            res = conn.search_s(self.tree_dn, self.LDAP_SCOPE, query, attrs)
         except ldap.NO_SUCH_OBJECT:
             return None
+        finally:
+            conn.unbind_s()
         try:
             return res[0]
         except IndexError:
@@ -287,6 +364,8 @@ class BaseLdap(object):
                                  self.attribute_mapping.values())
         except ldap.NO_SUCH_OBJECT:
             return []
+        finally:
+            conn.unbind_s()
 
     def get(self, id, filter=None):
         res = self._ldap_get(id, filter)
@@ -320,16 +399,29 @@ class BaseLdap(object):
         for k, v in values.iteritems():
             if k == 'id' or k in self.attribute_ignore:
                 continue
+
+            # attribute value has not changed
+            if k in old_obj and old_obj[k] == v:
+                continue
+
+            if k in self.immutable_attrs:
+                msg = (_("Cannot change %(option_name)s %(attr)s") %
+                       {'option_name': self.options_name, 'attr': k})
+                raise exception.ValidationError(msg)
+
             if v is None:
-                if old_obj[k] is not None:
+                if old_obj.get(k) is not None:
                     modlist.append((ldap.MOD_DELETE,
                                     self.attribute_mapping.get(k, k),
                                     None))
-            elif old_obj[k] != v:
-                if old_obj[k] is None:
-                    op = ldap.MOD_ADD
-                else:
-                    op = ldap.MOD_REPLACE
+                continue
+
+            current_value = old_obj.get(k)
+            if current_value is None:
+                op = ldap.MOD_ADD
+                modlist.append((op, self.attribute_mapping.get(k, k), [v]))
+            elif current_value != v:
+                op = ldap.MOD_REPLACE
                 modlist.append((op, self.attribute_mapping.get(k, k), [v]))
 
         if modlist:
@@ -338,6 +430,8 @@ class BaseLdap(object):
                 conn.modify_s(self._id_to_dn(id), modlist)
             except ldap.NO_SUCH_OBJECT:
                 raise self._not_found(id)
+            finally:
+                conn.unbind_s()
 
         return self.get(id)
 
@@ -351,6 +445,8 @@ class BaseLdap(object):
             conn.delete_s(self._id_to_dn(id))
         except ldap.NO_SUCH_OBJECT:
             raise self._not_found(id)
+        finally:
+            conn.unbind_s()
 
     def deleteTree(self, id):
         conn = self.get_connection()
@@ -362,38 +458,113 @@ class BaseLdap(object):
                               serverctrls=[tree_delete_control])
         except ldap.NO_SUCH_OBJECT:
             raise self._not_found(id)
+        finally:
+            conn.unbind_s()
 
 
 class LdapWrapper(object):
-    def __init__(self, url, page_size, alias_dereferencing=None):
+    def __init__(self, url, page_size, alias_dereferencing=None,
+                 use_tls=False, tls_cacertfile=None, tls_cacertdir=None,
+                 tls_req_cert='demand'):
         LOG.debug(_("LDAP init: url=%s"), url)
+        LOG.debug(_('LDAP init: use_tls=%(use_tls)s\n'
+                  'tls_cacertfile=%(tls_cacertfile)s\n'
+                  'tls_cacertdir=%(tls_cacertdir)s\n'
+                  'tls_req_cert=%(tls_req_cert)s\n'
+                  'tls_avail=%(tls_avail)s\n') %
+                  {'use_tls': use_tls,
+                   'tls_cacertfile': tls_cacertfile,
+                   'tls_cacertdir': tls_cacertdir,
+                   'tls_req_cert': tls_req_cert,
+                   'tls_avail': ldap.TLS_AVAIL
+                   })
+
+        #NOTE(topol)
+        #for extra debugging uncomment the following line
+        #ldap.set_option(ldap.OPT_DEBUG_LEVEL, 4095)
+
+        using_ldaps = url.lower().startswith("ldaps")
+
+        if use_tls and using_ldaps:
+            raise AssertionError(_('Invalid TLS / LDAPS combination'))
+
+        if use_tls:
+            if not ldap.TLS_AVAIL:
+                raise ValueError(_('Invalid LDAP TLS_AVAIL option: %s. TLS '
+                                   'not available') % ldap.TLS_AVAIL)
+            if tls_cacertfile:
+                #NOTE(topol)
+                #python ldap TLS does not verify CACERTFILE or CACERTDIR
+                #so we add some extra simple sanity check verification
+                #Also, setting these values globally (i.e. on the ldap object)
+                #works but these values are ignored when setting them on the
+                #connection
+                if not os.path.isfile(tls_cacertfile):
+                    raise IOError(_("tls_cacertfile %s not found "
+                                    "or is not a file") %
+                                  tls_cacertfile)
+                ldap.set_option(ldap.OPT_X_TLS_CACERTFILE, tls_cacertfile)
+            elif tls_cacertdir:
+                #NOTE(topol)
+                #python ldap TLS does not verify CACERTFILE or CACERTDIR
+                #so we add some extra simple sanity check verification
+                #Also, setting these values globally (i.e. on the ldap object)
+                #works but these values are ignored when setting them on the
+                #connection
+                if not os.path.isdir(tls_cacertdir):
+                    raise IOError(_("tls_cacertdir %s not found "
+                                    "or is not a directory") %
+                                  tls_cacertdir)
+                ldap.set_option(ldap.OPT_X_TLS_CACERTDIR, tls_cacertdir)
+            if tls_req_cert in LDAP_TLS_CERTS.values():
+                ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, tls_req_cert)
+            else:
+                LOG.debug(_("LDAP TLS: invalid TLS_REQUIRE_CERT Option=%s"),
+                          tls_req_cert)
+
         self.conn = ldap.initialize(url)
+        self.conn.protocol_version = ldap.VERSION3
+
         if alias_dereferencing is not None:
             self.conn.set_option(ldap.OPT_DEREF, alias_dereferencing)
         self.page_size = page_size
 
+        if use_tls:
+            self.conn.start_tls_s()
+
     def simple_bind_s(self, user, password):
         LOG.debug(_("LDAP bind: dn=%s"), user)
         return self.conn.simple_bind_s(user, password)
 
+    def unbind_s(self):
+        LOG.debug("LDAP unbind")
+        return self.conn.unbind_s()
+
     def add_s(self, dn, attrs):
         ldap_attrs = [(kind, [py2ldap(x) for x in safe_iter(values)])
                       for kind, values in attrs]
-        if LOG.isEnabledFor(logging.DEBUG):
-            sane_attrs = [(kind, values
-                           if kind != 'userPassword'
-                           else ['****'])
-                          for kind, values in ldap_attrs]
-            LOG.debug(_('LDAP add: dn=%s, attrs=%s'), dn, sane_attrs)
+        sane_attrs = [(kind, values
+                       if kind != 'userPassword'
+                       else ['****'])
+                      for kind, values in ldap_attrs]
+        LOG.debug(_('LDAP add: dn=%(dn)s, attrs=%(attrs)s') % {
+            'dn': dn, 'attrs': sane_attrs})
         return self.conn.add_s(dn, ldap_attrs)
 
     def search_s(self, dn, scope, query, attrlist=None):
-        if LOG.isEnabledFor(logging.DEBUG):
-            LOG.debug(_('LDAP search: dn=%s, scope=%s, query=%s, attrs=%s'),
-                      dn,
-                      scope,
-                      query,
-                      attrlist)
+        # NOTE(morganfainberg): Remove "None" singletons from this list, which
+        # allows us to set mapped attributes to "None" as defaults in config.
+        # Without this filtering, the ldap query would raise a TypeError since
+        # attrlist is expected to be an iterable of strings.
+        if attrlist is not None:
+            attrlist = [attr for attr in attrlist if attr is not None]
+        LOG.debug(_(
+            'LDAP search: dn=%(dn)s, scope=%(scope)s, query=%(query)s, '
+            'attrs=%(attrlist)s') % {
+                'dn': dn,
+                'scope': scope,
+                'query': query,
+                'attrlist': attrlist})
         if self.page_size:
             res = self.paged_search_s(dn, scope, query, attrlist)
         else:
@@ -453,11 +624,11 @@ class LdapWrapper(object):
                         else [py2ldap(x) for x in safe_iter(values)]))
             for op, kind, values in modlist]
 
-        if LOG.isEnabledFor(logging.DEBUG):
-            sane_modlist = [(op, kind, (values if kind != 'userPassword'
-                                        else ['****']))
-                            for op, kind, values in ldap_modlist]
-            LOG.debug(_("LDAP modify: dn=%s, modlist=%s"), dn, sane_modlist)
+        sane_modlist = [(op, kind, (values if kind != 'userPassword'
+                                    else ['****']))
+                        for op, kind, values in ldap_modlist]
+        LOG.debug(_('LDAP modify: dn=%(dn)s, modlist=%(modlist)s') % {
+            'dn': dn, 'modlist': sane_modlist})
 
         return self.conn.modify_s(dn, ldap_modlist)
 
@@ -466,7 +637,9 @@ class LdapWrapper(object):
         return self.conn.delete_s(dn)
 
     def delete_ext_s(self, dn, serverctrls):
-        LOG.debug(_("LDAP delete_ext: dn=%s, serverctrls=%s"), dn, serverctrls)
+        LOG.debug(
+            _('LDAP delete_ext: dn=%(dn)s, serverctrls=%(serverctrls)s') % {
+                'dn': dn, 'serverctrls': serverctrls})
         return self.conn.delete_ext_s(dn, serverctrls)
 
     def _disable_paging(self):
@@ -508,11 +681,13 @@ class EnabledEmuMixIn(BaseLdap):
         try:
             enabled_value = conn.search_s(self.enabled_emulation_dn,
                                           ldap.SCOPE_BASE,
-                                          query)
+                                          query, ['cn'])
         except ldap.NO_SUCH_OBJECT:
             return False
         else:
             return bool(enabled_value)
+        finally:
+            conn.unbind_s()
 
     def _add_enabled(self, object_id):
         if not self._get_enabled(object_id):
@@ -529,6 +704,8 @@ class EnabledEmuMixIn(BaseLdap):
                 if self.use_dumb_member:
                     attr_list[1][1].append(self.dumb_member)
                 conn.add_s(self.enabled_emulation_dn, attr_list)
+            finally:
+                conn.unbind_s()
 
     def _remove_enabled(self, object_id):
         conn = self.get_connection()
@@ -539,6 +716,8 @@ class EnabledEmuMixIn(BaseLdap):
             conn.modify_s(self.enabled_emulation_dn, modlist)
         except (ldap.NO_SUCH_OBJECT, ldap.NO_SUCH_ATTRIBUTE):
             pass
+        finally:
+            conn.unbind_s()
 
     def create(self, values):
         if self.enabled_emulation:
