@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,62 +16,36 @@
 
 """Extensions supporting OAuth1."""
 
-from oslo_config import cfg
-from oslo_serialization import jsonutils
-from oslo_utils import timeutils
-
 from keystone.common import controller
 from keystone.common import dependency
-from keystone.common import utils
 from keystone.common import wsgi
+from keystone import config
 from keystone.contrib.oauth1 import core as oauth1
-from keystone.contrib.oauth1 import validator
 from keystone import exception
-from keystone.i18n import _
-from keystone import notifications
+from keystone.openstack.common import jsonutils
+from keystone.openstack.common import timeutils
 
 
-CONF = cfg.CONF
+CONF = config.CONF
 
 
-@notifications.internal(notifications.INVALIDATE_USER_OAUTH_CONSUMER_TOKENS,
-                        resource_id_arg_index=0)
-def _emit_user_oauth_consumer_token_invalidate(payload):
-    # This is a special case notification that expect the payload to be a dict
-    # containing the user_id and the consumer_id. This is so that the token
-    # provider can invalidate any tokens in the token persistence if
-    # token persistence is enabled
-    pass
-
-
-@dependency.requires('oauth_api', 'token_provider_api')
+@dependency.requires('oauth_api', 'token_api')
 class ConsumerCrudV3(controller.V3Controller):
     collection_name = 'consumers'
     member_name = 'consumer'
 
-    @classmethod
-    def base_url(cls, context, path=None):
-        """Construct a path and pass it to V3Controller.base_url method."""
-
-        # NOTE(stevemar): Overriding path to /OS-OAUTH1/consumers so that
-        # V3Controller.base_url handles setting the self link correctly.
-        path = '/OS-OAUTH1/' + cls.collection_name
-        return controller.V3Controller.base_url(context, path=path)
-
     @controller.protected()
     def create_consumer(self, context, consumer):
         ref = self._assign_unique_id(self._normalize_dict(consumer))
-        initiator = notifications._get_request_audit_info(context)
-        consumer_ref = self.oauth_api.create_consumer(ref, initiator)
+        consumer_ref = self.oauth_api.create_consumer(ref)
         return ConsumerCrudV3.wrap_member(context, consumer_ref)
 
     @controller.protected()
     def update_consumer(self, context, consumer_id, consumer):
         self._require_matching_id(consumer_id, consumer)
         ref = self._normalize_dict(consumer)
-        self._validate_consumer_ref(ref)
-        initiator = notifications._get_request_audit_info(context)
-        ref = self.oauth_api.update_consumer(consumer_id, ref, initiator)
+        self._validate_consumer_ref(consumer)
+        ref = self.oauth_api.update_consumer(consumer_id, consumer)
         return ConsumerCrudV3.wrap_member(context, ref)
 
     @controller.protected()
@@ -84,12 +60,10 @@ class ConsumerCrudV3(controller.V3Controller):
 
     @controller.protected()
     def delete_consumer(self, context, consumer_id):
-        user_token_ref = utils.get_token_ref(context)
-        payload = {'user_id': user_token_ref.user_id,
-                   'consumer_id': consumer_id}
-        _emit_user_oauth_consumer_token_invalidate(payload)
-        initiator = notifications._get_request_audit_info(context)
-        self.oauth_api.delete_consumer(consumer_id, initiator)
+        user_token_ref = self.token_api.get_token(context['token_id'])
+        user_id = user_token_ref['user'].get('id')
+        self.token_api.delete_tokens(user_id, consumer_id=consumer_id)
+        self.oauth_api.delete_consumer(consumer_id)
 
     def _validate_consumer_ref(self, consumer):
         if 'secret' in consumer:
@@ -102,71 +76,51 @@ class AccessTokenCrudV3(controller.V3Controller):
     collection_name = 'access_tokens'
     member_name = 'access_token'
 
-    @classmethod
-    def _add_self_referential_link(cls, context, ref):
-        # NOTE(lwolf): overriding method to add proper path to self link
-        ref.setdefault('links', {})
-        path = '/users/%(user_id)s/OS-OAUTH1/access_tokens' % {
-            'user_id': cls._get_user_id(ref)
-        }
-        ref['links']['self'] = cls.base_url(context, path) + '/' + ref['id']
-
     @controller.protected()
     def get_access_token(self, context, user_id, access_token_id):
         access_token = self.oauth_api.get_access_token(access_token_id)
         if access_token['authorizing_user_id'] != user_id:
             raise exception.NotFound()
-        access_token = self._format_token_entity(context, access_token)
+        access_token = self._format_token_entity(access_token)
         return AccessTokenCrudV3.wrap_member(context, access_token)
 
     @controller.protected()
     def list_access_tokens(self, context, user_id):
-        auth_context = context.get('environment',
-                                   {}).get('KEYSTONE_AUTH_CONTEXT', {})
-        if auth_context.get('is_delegated_auth'):
-            raise exception.Forbidden(
-                _('Cannot list request tokens'
-                  ' with a token issued via delegation.'))
         refs = self.oauth_api.list_access_tokens(user_id)
-        formatted_refs = ([self._format_token_entity(context, x)
-                           for x in refs])
+        formatted_refs = ([self._format_token_entity(x) for x in refs])
         return AccessTokenCrudV3.wrap_collection(context, formatted_refs)
 
     @controller.protected()
     def delete_access_token(self, context, user_id, access_token_id):
         access_token = self.oauth_api.get_access_token(access_token_id)
         consumer_id = access_token['consumer_id']
-        payload = {'user_id': user_id, 'consumer_id': consumer_id}
-        _emit_user_oauth_consumer_token_invalidate(payload)
-        initiator = notifications._get_request_audit_info(context)
+        self.token_api.delete_tokens(user_id, consumer_id=consumer_id)
         return self.oauth_api.delete_access_token(
-            user_id, access_token_id, initiator)
+            user_id, access_token_id)
 
-    @staticmethod
-    def _get_user_id(entity):
-        return entity.get('authorizing_user_id', '')
-
-    def _format_token_entity(self, context, entity):
+    def _format_token_entity(self, entity):
 
         formatted_entity = entity.copy()
         access_token_id = formatted_entity['id']
-        user_id = self._get_user_id(formatted_entity)
+        user_id = ""
         if 'role_ids' in entity:
             formatted_entity.pop('role_ids')
         if 'access_secret' in entity:
             formatted_entity.pop('access_secret')
+        if 'authorizing_user_id' in entity:
+            user_id = formatted_entity['authorizing_user_id']
 
         url = ('/users/%(user_id)s/OS-OAUTH1/access_tokens/%(access_token_id)s'
                '/roles' % {'user_id': user_id,
                            'access_token_id': access_token_id})
 
         formatted_entity.setdefault('links', {})
-        formatted_entity['links']['roles'] = (self.base_url(context, url))
+        formatted_entity['links']['roles'] = (self.base_url(url))
 
         return formatted_entity
 
 
-@dependency.requires('oauth_api', 'role_api')
+@dependency.requires('oauth_api', 'assignment_api')
 class AccessTokenRolesV3(controller.V3Controller):
     collection_name = 'roles'
     member_name = 'role'
@@ -196,7 +150,7 @@ class AccessTokenRolesV3(controller.V3Controller):
         raise exception.RoleNotFound(_('Could not find role'))
 
     def _format_role_entity(self, role_id):
-        role = self.role_api.get_role(role_id)
+        role = self.assignment_api.get_role(role_id)
         formatted_entity = role.copy()
         if 'description' in role:
             formatted_entity.pop('description')
@@ -205,8 +159,8 @@ class AccessTokenRolesV3(controller.V3Controller):
         return formatted_entity
 
 
-@dependency.requires('assignment_api', 'oauth_api',
-                     'resource_api', 'token_provider_api')
+@dependency.requires('oauth_api', 'token_api', 'identity_api',
+                     'token_provider_api', 'assignment_api')
 class OAuthControllerV3(controller.V3Controller):
     collection_name = 'not_used'
     member_name = 'not_used'
@@ -216,7 +170,6 @@ class OAuthControllerV3(controller.V3Controller):
         oauth_headers = oauth1.get_oauth_headers(headers)
         consumer_id = oauth_headers.get('oauth_consumer_key')
         requested_project_id = headers.get('Requested-Project-Id')
-
         if not consumer_id:
             raise exception.ValidationError(
                 attribute='oauth_consumer_key', target='request')
@@ -224,33 +177,32 @@ class OAuthControllerV3(controller.V3Controller):
             raise exception.ValidationError(
                 attribute='requested_project_id', target='request')
 
-        # NOTE(stevemar): Ensure consumer and requested project exist
-        self.resource_api.get_project(requested_project_id)
-        self.oauth_api.get_consumer(consumer_id)
+        consumer_ref = self.oauth_api.get_consumer_with_secret(consumer_id)
+        consumer = oauth1.Consumer(key=consumer_ref['id'],
+                                   secret=consumer_ref['secret'])
 
-        url = self.base_url(context, context['path'])
-
-        req_headers = {'Requested-Project-Id': requested_project_id}
-        req_headers.update(headers)
-        request_verifier = oauth1.RequestTokenEndpoint(
-            request_validator=validator.OAuthValidator(),
-            token_generator=oauth1.token_generator)
-        h, b, s = request_verifier.create_request_token_response(
-            url,
+        url = oauth1.rebuild_url(context['path'])
+        oauth_request = oauth1.Request.from_request(
             http_method='POST',
-            body=context['query_string'],
-            headers=req_headers)
+            http_url=url,
+            headers=context['headers'],
+            query_string=context['query_string'],
+            parameters={'requested_project_id': requested_project_id})
+        oauth_server = oauth1.Server()
+        oauth_server.add_signature_method(oauth1.SignatureMethod_HMAC_SHA1())
+        params = oauth_server.verify_request(oauth_request,
+                                             consumer,
+                                             token=None)
 
-        if (not b) or int(s) > 399:
-            msg = _('Invalid signature')
+        project_params = params['requested_project_id']
+        if project_params != requested_project_id:
+            msg = _('Non-oauth parameter - project, do not match')
             raise exception.Unauthorized(message=msg)
 
         request_token_duration = CONF.oauth1.request_token_duration
-        initiator = notifications._get_request_audit_info(context)
         token_ref = self.oauth_api.create_request_token(consumer_id,
                                                         requested_project_id,
-                                                        request_token_duration,
-                                                        initiator)
+                                                        request_token_duration)
 
         result = ('oauth_token=%(key)s&oauth_token_secret=%(secret)s'
                   % {'key': token_ref['id'],
@@ -284,6 +236,7 @@ class OAuthControllerV3(controller.V3Controller):
             raise exception.ValidationError(
                 attribute='oauth_verifier', target='request')
 
+        consumer = self.oauth_api.get_consumer_with_secret(consumer_id)
         req_token = self.oauth_api.get_request_token(
             request_token_id)
 
@@ -295,17 +248,24 @@ class OAuthControllerV3(controller.V3Controller):
             if now > expires:
                 raise exception.Unauthorized(_('Request token is expired'))
 
-        url = self.base_url(context, context['path'])
+        consumer_obj = oauth1.Consumer(key=consumer['id'],
+                                       secret=consumer['secret'])
+        req_token_obj = oauth1.Token(key=req_token['id'],
+                                     secret=req_token['request_secret'])
+        req_token_obj.set_verifier(oauth_verifier)
 
-        access_verifier = oauth1.AccessTokenEndpoint(
-            request_validator=validator.OAuthValidator(),
-            token_generator=oauth1.token_generator)
-        h, b, s = access_verifier.create_access_token_response(
-            url,
+        url = oauth1.rebuild_url(context['path'])
+        oauth_request = oauth1.Request.from_request(
             http_method='POST',
-            body=context['query_string'],
-            headers=headers)
-        params = oauth1.extract_non_oauth_params(b)
+            http_url=url,
+            headers=context['headers'],
+            query_string=context['query_string'])
+        oauth_server = oauth1.Server()
+        oauth_server.add_signature_method(oauth1.SignatureMethod_HMAC_SHA1())
+        params = oauth_server.verify_request(oauth_request,
+                                             consumer_obj,
+                                             token=req_token_obj)
+
         if len(params) != 0:
             msg = _('There should not be any non-oauth parameters')
             raise exception.Unauthorized(message=msg)
@@ -327,10 +287,8 @@ class OAuthControllerV3(controller.V3Controller):
             raise exception.Unauthorized(message=msg)
 
         access_token_duration = CONF.oauth1.access_token_duration
-        initiator = notifications._get_request_audit_info(context)
         token_ref = self.oauth_api.create_access_token(request_token_id,
-                                                       access_token_duration,
-                                                       initiator)
+                                                       access_token_duration)
 
         result = ('oauth_token=%(key)s&oauth_token_secret=%(secret)s'
                   % {'key': token_ref['id'],
@@ -356,12 +314,6 @@ class OAuthControllerV3(controller.V3Controller):
         there is not another easy way to make sure the user knows which roles
         are being requested before authorizing.
         """
-        auth_context = context.get('environment',
-                                   {}).get('KEYSTONE_AUTH_CONTEXT', {})
-        if auth_context.get('is_delegated_auth'):
-            raise exception.Forbidden(
-                _('Cannot authorize a request token'
-                  ' with a token issued via delegation.'))
 
         req_token = self.oauth_api.get_request_token(request_token_id)
 
@@ -379,8 +331,8 @@ class OAuthControllerV3(controller.V3Controller):
             authed_roles.add(role['id'])
 
         # verify the authorizing user has the roles
-        user_token = utils.get_token_ref(context)
-        user_id = user_token.user_id
+        user_token = self.token_api.get_token(context['token_id'])
+        user_id = user_token['user'].get('id')
         project_id = req_token['requested_project_id']
         user_roles = self.assignment_api.get_roles_for_user_and_project(
             user_id, project_id)
@@ -396,10 +348,12 @@ class OAuthControllerV3(controller.V3Controller):
         # verify the user has the project too
         req_project_id = req_token['requested_project_id']
         user_projects = self.assignment_api.list_projects_for_user(user_id)
+        found = False
         for user_project in user_projects:
             if user_project['id'] == req_project_id:
+                found = True
                 break
-        else:
+        if not found:
             msg = _("User is not a member of the requested project")
             raise exception.Unauthorized(message=msg)
 
